@@ -11,8 +11,9 @@ build_time_vars = {'ABIFLAGS': 'm',
  'BINDIR': '/usr/bin',
  'BINLIBDEST': '/usr/lib/python3.6',
  'BLDLIBRARY': '-L. -lpython3.6m',
- 'BLDSHARED': 'gcc -pthread -shared -Wl,-O1,--sort-common,--as-needed,-z,relro '
-              '-Wl,-O1,--sort-common,--as-needed,-z,relro',
+ 'BLDSHARED': 'gcc -pthread -shared '
+              '-Wl,-O1,--sort-common,--as-needed,-z,relro,-z,now '
+              '-Wl,-O1,--sort-common,--as-needed,-z,relro,-z,now',
  'BUILDEXE': '',
  'BUILDPYTHON': 'python',
  'BUILD_GNU_TYPE': 'aarch64-unknown-linux-gnu',
@@ -21,22 +22,21 @@ build_time_vars = {'ABIFLAGS': 'm',
  'CCSHARED': '-fPIC',
  'CFLAGS': '-Wno-unused-result -Wsign-compare -DNDEBUG -g -fwrapv -O3 -Wall '
            '-Wstrict-prototypes -march=armv8-a -O2 -pipe '
-           '-fstack-protector-strong --param=ssp-buffer-size=4 -march=armv8-a '
-           '-O2 -pipe -fstack-protector-strong --param=ssp-buffer-size=4 '
-           '-march=armv8-a -O2 -pipe -fstack-protector-strong '
-           '--param=ssp-buffer-size=4',
+           '-fstack-protector-strong -fno-plt -march=armv8-a -O2 -pipe '
+           '-fstack-protector-strong -fno-plt -march=armv8-a -O2 -pipe '
+           '-fstack-protector-strong -fno-plt',
  'CFLAGSFORSHARED': '-fPIC',
  'CFLAGS_ALIASING': '',
  'CFLAGS_NODIST': '',
  'CONFIGFILES': 'configure configure.ac acconfig.h pyconfig.h.in '
                 'Makefile.pre.in',
  'CONFIGURE_CFLAGS': '-march=armv8-a -O2 -pipe -fstack-protector-strong '
-                     '--param=ssp-buffer-size=4',
+                     '-fno-plt',
  'CONFIGURE_CFLAGS_NODIST': '-std=c99 -Wextra -Wno-unused-result '
                             '-Wno-unused-parameter '
                             '-Wno-missing-field-initializers',
  'CONFIGURE_CPPFLAGS': '-D_FORTIFY_SOURCE=2',
- 'CONFIGURE_LDFLAGS': '-Wl,-O1,--sort-common,--as-needed,-z,relro',
+ 'CONFIGURE_LDFLAGS': '-Wl,-O1,--sort-common,--as-needed,-z,relro,-z,now',
  'CONFIG_ARGS': "'--prefix=/usr' '--enable-shared' '--with-threads' "
                 "'--with-computed-gotos' '--enable-optimizations' "
                 "'--without-lto' '--enable-ipv6' '--with-system-expat' "
@@ -44,14 +44,14 @@ build_time_vars = {'ABIFLAGS': 'm',
                 "'--with-system-libmpdec' "
                 "'--enable-loadable-sqlite-extensions' '--without-ensurepip' "
                 "'CFLAGS=-march=armv8-a -O2 -pipe -fstack-protector-strong "
-                "--param=ssp-buffer-size=4' "
-                "'LDFLAGS=-Wl,-O1,--sort-common,--as-needed,-z,relro' "
+                "-fno-plt' "
+                "'LDFLAGS=-Wl,-O1,--sort-common,--as-needed,-z,relro,-z,now' "
                 "'CPPFLAGS=-D_FORTIFY_SOURCE=2'",
  'CONFINCLUDEDIR': '/usr/include',
  'CONFINCLUDEPY': '/usr/include/python3.6m',
  'COREPYTHONPATH': ':',
- 'COVERAGE_INFO': '/build/python/src/Python-3.6.2/coverage.info',
- 'COVERAGE_REPORT': '/build/python/src/Python-3.6.2/lcov-report',
+ 'COVERAGE_INFO': '/build/python/src/Python-3.6.3/coverage.info',
+ 'COVERAGE_REPORT': '/build/python/src/Python-3.6.3/lcov-report',
  'COVERAGE_REPORT_OPTIONS': '--no-branch-coverage --title "CPython lcov '
                             'report"',
  'CPPFLAGS': '-I. -I./Include -D_FORTIFY_SOURCE=2 -D_FORTIFY_SOURCE=2',
@@ -424,6 +424,7 @@ build_time_vars = {'ABIFLAGS': 'm',
  'HAVE_SYS_STATVFS_H': 1,
  'HAVE_SYS_STAT_H': 1,
  'HAVE_SYS_SYSCALL_H': 1,
+ 'HAVE_SYS_SYSMACROS_H': 1,
  'HAVE_SYS_SYS_DOMAIN_H': 0,
  'HAVE_SYS_TERMIO_H': 0,
  'HAVE_SYS_TIMES_H': 1,
@@ -485,13 +486,14 @@ build_time_vars = {'ABIFLAGS': 'm',
  'IO_H': 'Modules/_io/_iomodule.h',
  'IO_OBJS': '\\',
  'LDCXXSHARED': 'g++ -pthread -shared',
- 'LDFLAGS': '-Wl,-O1,--sort-common,--as-needed,-z,relro '
-            '-Wl,-O1,--sort-common,--as-needed,-z,relro',
+ 'LDFLAGS': '-Wl,-O1,--sort-common,--as-needed,-z,relro,-z,now '
+            '-Wl,-O1,--sort-common,--as-needed,-z,relro,-z,now',
  'LDLAST': '',
  'LDLIBRARY': 'libpython3.6m.so',
  'LDLIBRARYDIR': '',
- 'LDSHARED': 'gcc -pthread -shared -Wl,-O1,--sort-common,--as-needed,-z,relro '
-             '-Wl,-O1,--sort-common,--as-needed,-z,relro',
+ 'LDSHARED': 'gcc -pthread -shared '
+             '-Wl,-O1,--sort-common,--as-needed,-z,relro,-z,now '
+             '-Wl,-O1,--sort-common,--as-needed,-z,relro,-z,now',
  'LDVERSION': '3.6m',
  'LIBC': '',
  'LIBDEST': '/usr/lib/python3.6',
@@ -585,26 +587,24 @@ build_time_vars = {'ABIFLAGS': 'm',
  'PYTHON_OBJS': '\\',
  'PY_CFLAGS': '-Wno-unused-result -Wsign-compare -DNDEBUG -g -fwrapv -O3 -Wall '
               '-Wstrict-prototypes -march=armv8-a -O2 -pipe '
-              '-fstack-protector-strong --param=ssp-buffer-size=4 '
-              '-march=armv8-a -O2 -pipe -fstack-protector-strong '
-              '--param=ssp-buffer-size=4 -march=armv8-a -O2 -pipe '
-              '-fstack-protector-strong --param=ssp-buffer-size=4',
+              '-fstack-protector-strong -fno-plt -march=armv8-a -O2 -pipe '
+              '-fstack-protector-strong -fno-plt -march=armv8-a -O2 -pipe '
+              '-fstack-protector-strong -fno-plt',
  'PY_CFLAGS_NODIST': '-std=c99 -Wextra -Wno-unused-result '
                      '-Wno-unused-parameter -Wno-missing-field-initializers',
  'PY_CORE_CFLAGS': '-Wno-unused-result -Wsign-compare -DNDEBUG -g -fwrapv -O3 '
                    '-Wall -Wstrict-prototypes -march=armv8-a -O2 -pipe '
-                   '-fstack-protector-strong --param=ssp-buffer-size=4 '
-                   '-march=armv8-a -O2 -pipe -fstack-protector-strong '
-                   '--param=ssp-buffer-size=4 -march=armv8-a -O2 -pipe '
-                   '-fstack-protector-strong --param=ssp-buffer-size=4 '
-                   '-std=c99 -Wextra -Wno-unused-result -Wno-unused-parameter '
+                   '-fstack-protector-strong -fno-plt -march=armv8-a -O2 -pipe '
+                   '-fstack-protector-strong -fno-plt -march=armv8-a -O2 -pipe '
+                   '-fstack-protector-strong -fno-plt -std=c99 -Wextra '
+                   '-Wno-unused-result -Wno-unused-parameter '
                    '-Wno-missing-field-initializers -I. -I./Include '
                    '-D_FORTIFY_SOURCE=2 -D_FORTIFY_SOURCE=2 -fPIC '
                    '-DPy_BUILD_CORE',
  'PY_CPPFLAGS': '-I. -I./Include -D_FORTIFY_SOURCE=2 -D_FORTIFY_SOURCE=2',
  'PY_FORMAT_SIZE_T': '"z"',
- 'PY_LDFLAGS': '-Wl,-O1,--sort-common,--as-needed,-z,relro '
-               '-Wl,-O1,--sort-common,--as-needed,-z,relro',
+ 'PY_LDFLAGS': '-Wl,-O1,--sort-common,--as-needed,-z,relro,-z,now '
+               '-Wl,-O1,--sort-common,--as-needed,-z,relro,-z,now',
  'Py_DEBUG': 0,
  'Py_ENABLE_SHARED': 1,
  'Py_HASH_ALGORITHM': 0,
@@ -613,7 +613,7 @@ build_time_vars = {'ABIFLAGS': 'm',
  'READELF': 'readelf',
  'RESSRCDIR': 'Mac/Resources/framework',
  'RETSIGTYPE': 'void',
- 'RUNSHARED': 'LD_LIBRARY_PATH=/build/python/src/Python-3.6.2',
+ 'RUNSHARED': 'LD_LIBRARY_PATH=/build/python/src/Python-3.6.3',
  'SCRIPTDIR': '/usr/lib',
  'SETPGRP_HAVE_ARG': 0,
  'SGI_ABI': '',
@@ -654,9 +654,9 @@ build_time_vars = {'ABIFLAGS': 'm',
  'TCLTK_LIBS': '',
  'TESTOPTS': '',
  'TESTPATH': '',
- 'TESTPYTHON': 'LD_LIBRARY_PATH=/build/python/src/Python-3.6.2 ./python',
+ 'TESTPYTHON': 'LD_LIBRARY_PATH=/build/python/src/Python-3.6.3 ./python',
  'TESTPYTHONOPTS': '',
- 'TESTRUNNER': 'LD_LIBRARY_PATH=/build/python/src/Python-3.6.2 ./python '
+ 'TESTRUNNER': 'LD_LIBRARY_PATH=/build/python/src/Python-3.6.3 ./python '
                './Tools/scripts/run_tests.py',
  'TESTTIMEOUT': 1200,
  'THREADOBJ': 'Python/thread.o',
@@ -680,8 +680,8 @@ build_time_vars = {'ABIFLAGS': 'm',
  'WITH_VALGRIND': 0,
  'X87_DOUBLE_ROUNDING': 0,
  'XMLLIBSUBDIRS': 'xml xml/dom xml/etree xml/parsers xml/sax',
- 'abs_builddir': '/build/python/src/Python-3.6.2',
- 'abs_srcdir': '/build/python/src/Python-3.6.2',
+ 'abs_builddir': '/build/python/src/Python-3.6.3',
+ 'abs_srcdir': '/build/python/src/Python-3.6.3',
  'datarootdir': '/usr/share',
  'exec_prefix': '/usr',
  'prefix': '/usr',
